@@ -294,7 +294,7 @@ find_group_data (const char *name, gid_t gid, const char *use_service)
 
   nsw_free (nswp);
 
-  if (nss_handle)
+  if (data->service != S_LOCAL && data->service != S_NONE)
     dlclose (nss_handle);
 
   return data;
