@@ -37,6 +37,9 @@
 #include <sys/resource.h>
 #include <security/pam_appl.h>
 #include <security/pam_misc.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #include "error_codes.h"
 #include "public.h"
@@ -424,6 +427,7 @@ main (int argc, char *argv[])
 
       if (!pw)
 	{
+	  sec_log (program, MSG_NO_ACCOUNT_FOUND, uid);
 	  fprintf (stderr, _("%s: Cannot determine your user name.\n"),
 		   program);
 	  return E_UNKNOWN_USER;
@@ -720,7 +724,9 @@ main (int argc, char *argv[])
     }
   else
     {
+#ifdef HAVE_NSCD_FLUSH_CACHE
       nscd_flush_cache ("passwd");
+#endif
       if (!silent)
 	printf (_("Aging information changed.\n"));
     }
