@@ -35,6 +35,9 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/resource.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -424,6 +427,7 @@ main (int argc, char **argv)
 
       if (!pw)
 	{
+	  sec_log (program, MSG_NO_ACCOUNT_FOUND, getuid ());
 	  fprintf (stderr, _("%s: Cannot determine your user name.\n"),
 		   program);
 	  return E_UNKNOWN_USER;
@@ -562,15 +566,16 @@ main (int argc, char **argv)
 		   (unsigned int) gr_data.gr.gr_gid, getuid());
 	}
 
+#ifdef HAVE_NSCD_FLUSH_CACHE
       /* flush NSCD cache.  */
       nscd_flush_cache ("group");
+#endif
 
       if (use_service == NULL || strcmp (use_service, "files") == 0)
 	ulckpwdf ();
 
-      /* XXX sec_log calling of call_script */
       i = call_script ("GROUPADD_CMD", gr_data.gr.gr_name,
-		       gr_data.gr.gr_gid, getuid (), NULL);
+		       gr_data.gr.gr_gid, getuid (), NULL, program);
       if (i != 0)
 	{
 	  fprintf (stderr, _("%s: GROUPADD_CMD fails with exit code %d.\n"),
