@@ -36,6 +36,9 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/resource.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -47,6 +50,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "logging.h"
 #include "utf8conv.h"
 #include "logindefs.h"
 #include "read-files.h"
@@ -134,6 +138,8 @@ is_primary_group (gid_t gid, int have_extrapath)
 		   _("%s: GID `%u' is primary group of `%s'.\n"),
 		   program, (unsigned int) gid,
 		   utf8_to_locale (pw->pw_name));
+	  sec_log (program, MSG_GID_IS_PRIMARY_GROUP, (unsigned int) gid,
+		   pw->pw_name, getuid ());
 	  retval = 1;
 	}
     }
@@ -149,6 +155,8 @@ is_primary_group (gid_t gid, int have_extrapath)
 		   _("%s: GID `%u' is primary group of `%s'.\n"),
 		   program, (unsigned int) gid,
 		   utf8_to_locale (pw->pw_name));
+	  sec_log (program, MSG_GID_IS_PRIMARY_GROUP, (unsigned int) gid,
+		   pw->pw_name, getuid ());
 	  retval = 1;
 	}
       }
@@ -177,6 +185,8 @@ main (int argc, char **argv)
 
   init_environment ();
 
+  open_sec_log ();
+
   while (1)
     {
       int c;
@@ -270,13 +280,18 @@ main (int argc, char **argv)
 
       if (!pw)
 	{
+	  sec_log (program, MSG_NO_ACCOUNT_FOUND, getuid ());
 	  fprintf (stderr, _("%s: Cannot determine your user name.\n"),
 		   program);
 	  return E_NOTFOUND;
 	}
 
       if (do_authentication ("shadow", pw->pw_name, NULL) != 0)
-	return E_NOPERM;
+	{
+          sec_log (program, MSG_PERMISSION_DENIED, pw->pw_name,
+                   pw->pw_uid, getuid());
+          return E_NOPERM;
+        }
     }
 
   remove_group = locale_to_utf8 (argv[0]);
@@ -285,11 +300,14 @@ main (int argc, char **argv)
   if (gr_data == NULL || gr_data->service == S_NONE)
     {
       if (use_service)
-	fprintf (stderr, _("%s: Group `%s' is not known to service `%s'.\n"),
+	fprintf (stderr, _("%s: Group `%s' not found in service `%s'.\n"),
 		 program, utf8_to_locale (remove_group), use_service);
       else
 	fprintf (stderr, _("%s: Unknown group `%s'.\n"), program,
 		 utf8_to_locale (remove_group));
+
+      sec_log (program, MSG_UNKNOWN_GROUP, remove_group, getuid ());
+
       return E_NOTFOUND;
     }
 
@@ -297,6 +315,8 @@ main (int argc, char **argv)
     {
       fprintf (stderr, _("%s: Cannot remove user's primary group.\n"),
 	       program);
+      sec_log (program, MSG_CANNOT_REMOVE_PRIMARY_GROUP,
+	       gr_data->gr.gr_name, getuid());
       return E_GROUP_BUSY;
     }
 
@@ -349,6 +369,7 @@ main (int argc, char **argv)
   if (gr_data->service == S_LOCAL && lock_database () != 0)
     {
       fputs (_("Cannot lock group file: already locked.\n"), stderr);
+      sec_log (program, MSG_GROUP_FILE_ALREADY_LOCKED);
       return E_PWDBUSY;
     }
 
@@ -357,12 +378,19 @@ main (int argc, char **argv)
     {
       fprintf (stderr, _("%s: Error deleting group `%s'.\n"),
 	       program, utf8_to_locale (gr_data->gr.gr_name));
+      sec_log (program, MSG_ERROR_REMOVING_GROUP, gr_data->gr.gr_name,
+	       gr_data->gr.gr_gid, getuid ());
       free_group_t (gr_data);
       return E_FAILURE;
     }
+  else
+    sec_log (program, MSG_GROUP_DELETED, gr_data->gr.gr_name,
+	     gr_data->gr.gr_gid, getuid ());
 
+#ifdef HAVE_NSCD_FLUSH_CACHE
   /* flush NSCD cache to remove group really from the system.  */
   nscd_flush_cache ("group");
+#endif
 
   if (gr_data->service == S_LOCAL)
     ulckpwdf ();
