@@ -36,6 +36,9 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/resource.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -47,6 +50,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "logging.h"
 #include "utf8conv.h"
 #include "logindefs.h"
 #include "read-files.h"
@@ -243,6 +247,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  open_sec_log ();
+
   /* Before going any further, raise the ulimit and ignore
      signals.  */
   init_environment ();
@@ -383,13 +389,18 @@ main (int argc, char **argv)
 
       if (!pw)
 	{
+	  sec_log (program, MSG_NO_ACCOUNT_FOUND, getuid ());
 	  fprintf (stderr, _("%s: Cannot determine your user name.\n"),
 		   program);
 	  return E_UNKNOWN_USER;
 	}
 
       if (do_authentication ("shadow", pw->pw_name, NULL) != 0)
-	return E_NOPERM;
+	{
+	  sec_log (program, MSG_PERMISSION_DENIED, pw->pw_name,
+                   pw->pw_uid, getuid());
+	  return E_NOPERM;
+	}
     }
 
   modify_group = locale_to_utf8 (argv[0]);
@@ -397,8 +408,15 @@ main (int argc, char **argv)
   gr_data = find_group_data (modify_group, 0, use_service);
   if (gr_data == NULL || gr_data->service == S_NONE)
     {				/* Group does not exist.  */
-      fprintf (stderr, _("%s: group `%s' does not exist.\n"),
-	       program, utf8_to_locale (modify_group));
+      if (use_service)
+        fprintf (stderr, _("%s: Group `%s' not found in service `%s'.\n"),
+                 program, utf8_to_locale (modify_group), use_service);
+      else
+        fprintf (stderr, _("%s: Unknown group `%s'.\n"), program,
+                 utf8_to_locale (modify_group));
+
+      sec_log (program, MSG_UNKNOWN_GROUP, modify_group, getuid ());
+
       return E_NOTFOUND;
     }
 
@@ -408,8 +426,9 @@ main (int argc, char **argv)
       if (getgrgid (new_gid) != NULL ||
 	  (have_extrapath && files_getgrgid (new_gid) != NULL))
 	{
-	  fprintf (stderr, _("%s: UID %u is not unique.\n"),
+	  fprintf (stderr, _("%s: GID %u is not unique.\n"),
 		   program, new_gid);
+	  sec_log (program, MSG_GID_NOT_UNIQUE, new_gid, getuid ());
 	  return E_GID_IN_USE;
 	}
     }
@@ -422,6 +441,7 @@ main (int argc, char **argv)
 	{
 	  fprintf (stderr, _("%s: Invalid group name `%s'.\n"),
 		   program, utf8_to_locale (new_name));
+	  sec_log (program, MSG_GROUP_NAME_INVALID, new_name, getuid())
 	  return E_BAD_ARG;
 	}
       else
@@ -431,6 +451,7 @@ main (int argc, char **argv)
 	    {
 	      fprintf (stderr, _("%s: Group `%s' already exists.\n"),
 		       program, utf8_to_locale (new_name));
+	      sec_log (program, MSG_GROUP_ALREADY_EXISTS, new_name, getuid ());
 	      return E_NAME_IN_USE;
 	    }
 	}
@@ -442,6 +463,7 @@ main (int argc, char **argv)
       lock_database () != 0)
     {
       fputs (_("Cannot lock password file: already locked.\n"), stderr);
+      sec_log (program, MSG_PASSWD_FILE_ALREADY_LOCKED);
       return E_PWDBUSY;
     }
   else
@@ -500,10 +522,34 @@ main (int argc, char **argv)
 #endif
 
       if (write_group_data (gr_data, 1) != 0)
-	return E_FAILURE;
+	{
+	  sec_log (program, MSG_ERROR_MODIFYING_GROUP,
+		   gr_data->gr.gr_name,  gr_data->gr.gr_gid, getuid())
+	    return E_FAILURE;
+        }
+      else
+        {
+	  if (remove_user)
+	    sec_log (program, MSG_USER_REMOVED_FROM_GROUP,
+		     remove_user, gr_data->gr.gr_name, gr_data->gr.gr_gid, getuid());
+	  if (add_user)
+	    sec_log (program, MSG_USER_ADDED_TO_GROUP, add_user, gr_data->gr.gr_name,
+		     gr_data->gr.gr_gid, getuid());
+	  if (new_name)
+	    sec_log (program, MSG_GROUP_NAME_CHANGED, gr_data->new_name,
+		     gr_data->gr.gr_name, gr_data->gr.gr_gid, getuid());
+	  if (new_password)
+	    sec_log (program, MSG_GROUP_PASSWORD_CHANGED, gr_data->gr.gr_name,
+		     gr_data->gr.gr_gid, getuid());
+	  if (know_gid)
+	    sec_log (program, MSG_GROUP_ID_CHANGED, gr_data->gr.gr_name, gr_data->new_gid,
+		     gr_data->gr.gr_gid, getuid());
+	}
 
+#ifdef HAVE_NSCD_FLUSH_CACHE
       /* flush NSCD cache, else later calls could get obsolete data.  */
       nscd_flush_cache ("group");
+#endif
     }
 
   if (use_service == NULL || strcmp (use_service, "files") == 0)
