@@ -36,6 +36,9 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/resource.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -47,6 +50,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "logging.h"
 #include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
@@ -825,6 +829,7 @@ find_free_uid (int is_system_account, int have_extrapath)
 
       if (userid == uid_max)
 	{
+	  sec_log (program, MSG_NO_FREE_UID, uid_min, uid_max);
 	  fprintf (stderr, _("%s: Can't get unique uid in range %u - %u.\n"),
 		   program, uid_min, uid_max);
 	  exit (E_FAILURE);
@@ -962,6 +967,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  open_sec_log ();
+
   /* Before going any further, raise the ulimit and ignore
      signals.  */
 
@@ -1277,13 +1284,18 @@ main (int argc, char **argv)
 
       if (!pw)
 	{
+	  sec_log (program, MSG_NO_ACCOUNT_FOUND, getuid ());
 	  fprintf (stderr, _("%s: Cannot determine your user name.\n"),
 		   program);
 	  return E_UNKNOWN_USER;
 	}
 
       if (do_authentication (program, pw->pw_name, NULL) != 0)
-	return E_NOPERM;
+	{
+	  sec_log (program, MSG_PERMISSION_DENIED, pw->pw_name,
+                   pw->pw_uid, getuid ());
+	  return E_NOPERM;
+	}
     }
 
   if (modify_defaults)
@@ -1299,10 +1311,17 @@ main (int argc, char **argv)
 
       if (write_defaults (useradd_default_file, &dflt) != 0)
 	{
+	  sec_log (program, MSG_UPDATING_DEFAULT_CONFIG_FAILED,
+		   useradd_default_file, getuid ());
 	  fprintf (stderr, _("%s: Writing of `%s' was not successful.\n"),
 		   program, useradd_default_file);
 	  return E_LOGIN_DEFS;
 	}
+
+      sec_log (program, MSG_CONFIG_DEFAULTS_CHANGED,
+	       dflt.group, new_home, dflt.shell, dflt.inactive,
+	       date2str(DAY*dflt.expire), getuid ());
+
       return E_SUCCESS;
     }
 
@@ -1313,6 +1332,7 @@ main (int argc, char **argv)
       if (getpwuid (new_uid) != NULL ||
 	  (have_extrapath && files_getpwuid (new_uid) != NULL))
 	{
+	  sec_log (program, MSG_UID_NOT_UNIQUE, new_uid, getuid ());
 	  fprintf (stderr, _("%s: UID %u is not unique.\n"),
 		   program, new_uid);
 	  return E_UID_IN_USE;
@@ -1322,6 +1342,8 @@ main (int argc, char **argv)
   new_account = argv[0];
   if (check_name (new_account) != 0)
     {
+      sec_log (program, MSG_USER_NAME_INVALID,
+	       new_account, getuid ());
       fprintf (stderr, _("%s: Invalid user name `%s'.\n"),
 	       program, new_account);
       return E_BAD_ARG;
@@ -1335,12 +1357,14 @@ main (int argc, char **argv)
   if ((use_service == NULL || strcmp (use_service, "files") == 0) &&
       lock_database () != 0)
     {
+      sec_log (program, MSG_PASSWD_FILE_ALREADY_LOCKED);
       fputs (_("Cannot lock password file: already locked.\n"), stderr);
       return E_PWDBUSY;
     }
   else if (getpwnam (new_account) != NULL ||
 	   (have_extrapath && files_getpwnam (new_account) != NULL))
     {				/* User does already exists.  */
+      sec_log (program, MSG_USER_ALREADY_EXISTS, new_account, getuid ());
       fprintf (stderr, _("%s: User `%s' already exists.\n"),
 	       program, new_account);
       return E_NAME_IN_USE;
@@ -1405,11 +1429,10 @@ main (int argc, char **argv)
 	  pw_data.sp.sp_flag = -1;
 	}
 
-      syslog (LOG_INFO,
-	      "new user: name=%s, uid=%u, gid=%u, home=%s, shell=%s",
-	      pw_data.pw.pw_name, (unsigned int) pw_data.pw.pw_uid,
-	      (unsigned int) pw_data.pw.pw_gid, pw_data.pw.pw_dir,
-	      pw_data.pw.pw_shell);
+      sec_log (program, MSG_NEW_USER_ADDED,
+	       pw_data.pw.pw_name, (unsigned int) pw_data.pw.pw_uid,
+	       (unsigned int) pw_data.pw.pw_gid, pw_data.pw.pw_dir,
+	       pw_data.pw.pw_shell, getuid ());
 
       /* Clear old log entries, but only if this UID is not shared
          with another account.  */
@@ -1558,14 +1581,23 @@ main (int argc, char **argv)
 			   program, gr_data->gr.gr_name);
 		  retval = E_GRP_UPDATE;
 		}
+	      else
+		{
+		  sec_log (program, MSG_USER_ADDED_TO_GROUP,
+			   pw_data.pw.pw_name, pw_data.pw.pw_uid,
+			   gr_data->gr.gr_name, gr_data->gr.gr_gid,
+			   getuid ());
+		}
 	    }
 	  free_group_t (gr_data);
 	}
 
+#ifdef HAVE_NSCD_FLUSH_CACHE
       /* flush NSCD cache, else creating of home directory could fail
          because of unknown user.  */
       nscd_flush_cache ("passwd");
       nscd_flush_cache ("group");
+#endif
 
       if (create_homedir)
 	{
@@ -1578,13 +1610,16 @@ main (int argc, char **argv)
 		ulckpwdf ();
 	      return ret;
 	    }
+	  sec_log (program, MSG_HOME_DIR_CREATED,
+		   pw_data.pw.pw_name, pw_data.pw.pw_uid, new_home,
+		   getuid ());
 	}
 
       if (use_service == NULL || strcmp (use_service, "files") == 0)
 	ulckpwdf ();
 
       i = call_script ("USERADD_CMD", pw_data.pw.pw_name, pw_data.pw.pw_uid,
-		       pw_data.pw.pw_gid, pw_data.pw.pw_dir);
+		       pw_data.pw.pw_gid, pw_data.pw.pw_dir, program);
       if (i != 0)
 	{
 	  fprintf (stderr, _("%s: USERADD_CMD fails with exit code %d.\n"),
