@@ -36,6 +36,9 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/resource.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -47,6 +50,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "logging.h"
 #include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
@@ -347,11 +351,20 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath)
 					       gr_data->gr.gr_mem);
 	  if (write_group_data (gr_data, 1) != 0)
 	    {
+	      sec_log (program, MSG_ERROR_REMOVE_USER_FROM_GROUP,
+		       pw_data->pw.pw_name, pw_data->pw.pw_uid,
+		       gr_data->gr.gr_name, gr_data->gr.gr_gid, getuid ());
 	      fprintf (stderr,
 		       _("%s: User not removed from group `%s'.\n"),
 		       program, gr_data->gr.gr_name);
 	      retval = E_GRP_UPDATE;
 	    }
+	  else
+	    {
+	      sec_log (program, MSG_USER_REMOVED_FROM_GROUP,
+		       pw_data->pw.pw_name, gr_data->gr.gr_name,
+		       gr_data->gr.gr_gid, getuid ())
+	    }
 
 	  item = item->next;
 	}
@@ -416,6 +429,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  open_sec_log ();
+
   /* Before going any further, raise the ulimit and ignore
      signals.  */
 
@@ -526,7 +541,11 @@ main (int argc, char **argv)
 	}
 
       if (do_authentication ("shadow", pw->pw_name, NULL) != 0)
-	return E_NOPERM;
+	{
+	  sec_log (program, MSG_PERMISSION_DENIED, pw->pw_name,
+                   pw->pw_uid, getuid ());
+	  return E_NOPERM;
+	}
     }
 
   remove_user = argv[0];
@@ -534,6 +553,7 @@ main (int argc, char **argv)
   pw_data = do_getpwnam (remove_user, use_service);
   if (pw_data == NULL || pw_data->service == S_NONE)
     {
+      sec_log (program, MSG_UNKNOWN_USER, remove_user, getuid ());
       if (use_service)
 	fprintf (stderr, _("%s: User `%s' is not known to service `%s'.\n"),
 		 program, remove_user, use_service);
@@ -564,7 +584,7 @@ main (int argc, char **argv)
 #endif
 
   i = call_script ("USERDEL_PRECMD", pw_data->pw.pw_name, pw_data->pw.pw_uid,
-		   pw_data->pw.pw_gid, pw_data->pw.pw_dir);
+		   pw_data->pw.pw_gid, pw_data->pw.pw_dir, program);
   if (i != 0)
     {
       fprintf (stderr, _("%s: USERDEL_PRECMD fails with exit code %d.\n"),
@@ -576,6 +596,7 @@ main (int argc, char **argv)
      add the user a second time or a second user with the same uid.  */
   if (pw_data->service == S_LOCAL && lock_database () != 0)
     {
+      sec_log (program, MSG_PASSWD_FILE_ALREADY_LOCKED);
       fputs (_("Cannot lock password file: already locked.\n"), stderr);
       return E_PWDBUSY;
     }
@@ -591,8 +612,12 @@ main (int argc, char **argv)
       /* Remove the mail file only if owned by user or -f was given.  */
       ret = is_owned_by (cp, pw_data->pw.pw_uid);
       if (ret == 0 && !force_removal)
-	fprintf (stderr, _("%s: `%s' is not owned by `%s', not removed.\n"),
-		 program, cp, pw_data->pw.pw_name);
+	{
+	  sec_log (program, MSG_NOT_OWNED_BY_USER,
+		   cp, pw_data->pw.pw_name, pw_data->pw.pw_uid, getuid ());
+	  fprintf (stderr, _("%s: `%s' is not owned by `%s', not removed.\n"),
+		   program, cp, pw_data->pw.pw_name);
+	}
       else if (ret == 1 || (ret == 0 && force_removal))
 	{
 	  if (unlink (cp) == -1)
@@ -604,8 +629,13 @@ main (int argc, char **argv)
 	 not used by any other user or -f was given.  */
       ret = is_owned_by (pw_data->pw.pw_dir, pw_data->pw.pw_uid);
       if (ret == 0 && !force_removal)
-	fprintf (stderr, _("%s: `%s' is not owned by `%s', not removed.\n"),
-		 program, pw_data->pw.pw_dir, pw_data->pw.pw_name);
+	{
+	  sec_log (program, MSG_NOT_OWNED_BY_USER,
+		   pw_data->pw.pw_dir, pw_data->pw.pw_name,
+		   pw_data->pw.pw_uid, getuid ());
+	  fprintf (stderr, _("%s: `%s' is not owned by `%s', not removed.\n"),
+		   program, pw_data->pw.pw_dir, pw_data->pw.pw_name);
+	}
       else if (ret == 1 || (ret == 0 && force_removal))
 	{
 	  if (!in_use_by_other_users (pw_data->pw.pw_dir,
@@ -615,6 +645,12 @@ main (int argc, char **argv)
 	      if (remove_dir_rec (pw_data->pw.pw_dir) != 0)
 		fprintf (stderr, _("%s: warning: can't remove `%s': %s"),
 			 program, pw_data->pw.pw_dir, strerror (errno));
+	      else
+		{
+		  sec_log (program, MSG_HOMEDIR_REMOVED,
+			   pw_data->pw.pw_name, pw_data->pw.pw_uid,
+			   pw_data->pw.pw_dir, getuid ());
+		}
 	    }
 	  else
 	    fprintf (stderr, _("%s: directory `%s' not removed.\n"),
@@ -627,21 +663,28 @@ main (int argc, char **argv)
   pw_data->todo = DO_DELETE;
   if (write_user_data (pw_data, 1) != 0)
     {
+      sec_log (program, MSG_ERROR_REMOVING_USER,
+	       pw_data->pw.pw_name, pw_data->pw.pw_uid, getuid ());
       fprintf (stderr, _("%s: error deleting user `%s'.\n"),
 	       program, pw_data->pw.pw_name);
       free_user_t (pw_data);
       return E_FAILURE;
     }
+  else
+    sec_log (program, MSG_USER_DELETED,
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid, getuid ());
 
+#ifdef HAVE_NSCD_FLUSH_CACHE
   /* flush NSCD cache to remove user really from the system.  */
   nscd_flush_cache ("passwd");
   nscd_flush_cache ("group");
+#endif
 
   if (pw_data->service == S_LOCAL)
     ulckpwdf ();
 
   i = call_script ("USERDEL_POSTCMD", pw_data->pw.pw_name, pw_data->pw.pw_uid,
-		   pw_data->pw.pw_gid, pw_data->pw.pw_dir);
+		   pw_data->pw.pw_gid, pw_data->pw.pw_dir, program);
   if (i != 0)
     {
       fprintf (stderr, _("%s: USERDEL_POSTCMD fails with exit code %d.\n"),
