@@ -36,6 +36,9 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/resource.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -1353,8 +1356,10 @@ main (int argc, char **argv)
       if (write_user_data (pw_data, 1) != 0)
 	return E_FAILURE;
 
+#ifdef HAVE_NSCD_FLUSH_CACHE
       /* flush NSCD cache, else later calls could get obsolete data.  */
       nscd_flush_cache ("passwd");
+#endif
     }
 
   /* Change the login name in group entries, too.
